@@ -1,4 +1,5 @@
 from ftw.book.interfaces import IBook
+from ftw.book.interfaces import IWithinBookLayer
 from ftw.book.testing import LATEX_ZCML_LAYER
 from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
 from ftw.pdfgenerator.interfaces import ILaTeXLayout
@@ -6,7 +7,10 @@ from ftw.pdfgenerator.interfaces import ILaTeXView
 from ftw.testing import MockTestCase
 from simplelayout.base.interfaces import IBlockConfig
 from simplelayout.types.common.interfaces import IParagraph
+from zope.app.component.hooks import setSite
 from zope.component import getMultiAdapter
+from zope.component import getSiteManager
+from zope.i18n.interfaces import IUserPreferredLanguages
 from zope.interface import alsoProvides
 
 
@@ -15,22 +19,32 @@ class TestParagraphLaTeXView(MockTestCase):
     layer = LATEX_ZCML_LAYER
 
     def get_mocks(self):
+        request = self.create_dummy(getPreferredLanguages=lambda: [])
+        alsoProvides(request, IWithinBookLayer)
+        alsoProvides(request, IUserPreferredLanguages)
+
         # mock is not yet in replay mode, so we use another layout dummy..
-        layout_obj = self.create_dummy()
+        layout_obj = self.create_dummy(use_package=lambda pkg_name: None)
         alsoProvides(layout_obj, ILaTeXLayout)
-        converter = getMultiAdapter((object(), object(), layout_obj),
+        converter = getMultiAdapter((object(), request, layout_obj),
                                     IHTML2LaTeXConverter)
 
         context = self.providing_stub([IParagraph, IBlockConfig])
-        request = self.create_dummy()
+
         layout = self.providing_mock([ILaTeXLayout])
         self.expect(layout.get_converter()).result(converter).count(0, None)
 
         return context, request, layout
 
-    def create_image_mocks(self, image_layout, caption, uid):
+    def create_image_mocks(self, image_layout, caption, uid,
+                           floated=True):
         context, request, layout = self.get_mocks()
 
+        self.site = self.create_dummy(
+            REQUEST=request,
+            getSiteManager=getSiteManager)
+        setSite(self.site)
+
         image = self.create_dummy(get_size=lambda: 11, data='hello world')
         builder = self.mocker.mock()
 
@@ -38,7 +52,8 @@ class TestParagraphLaTeXView(MockTestCase):
         self.expect(builder.add_file('%s_image.jpg' % uid, image.data))
 
         self.expect(layout.use_package('graphicx'))
-        self.expect(layout.use_package('wrapfig'))
+        if floated:
+            self.expect(layout.use_package('wrapfig'))
 
         self.expect(context.getImage()).result(image).count(1, None)
         self.expect(context.image_layout).result(image_layout)
@@ -126,3 +141,47 @@ class TestParagraphLaTeXView(MockTestCase):
         self.assertIn(r'\includegraphics', imagepart)
         self.assertIn('123_image', imagepart)
         self.assertIn(r'Thats {\bf some} text.', textpart)
+
+    def test_heading_converts_to_bold(self):
+        # Within books the book structure (section, subsection, etc) is
+        # defined using chapters and paragraph titles. Therefore the
+        # text of the paragraph should not contain headings since it would
+        # result in inconsistent chapter numberings and other problems.
+
+        context, request, layout = self.get_mocks()
+
+        self.expect(context.getText()).result(
+            '<h1>the</h1> <h2>heading</h2> <h3>tags</h3> <h4>will</h4> ' + \
+                '<h5>be</h5> <h5>bold</h5>.')
+
+        self.replay()
+        view = getMultiAdapter((context, request, layout), ILaTeXView)
+        latex = view.get_text_latex()
+
+        self.assertEqual(
+            latex,
+            '{\\bf the} {\\bf heading} {\\bf tags} {\\bf will} ' + \
+                '{\\bf be} {\\bf bold}.\n')
+
+    def test_full_latex_rendering_not_floatable(self):
+        """Using "full" layout should not make a floatable image
+        (wrapfigure), even there is also text in the block.
+        """
+        paragraph, request, layout = self.create_image_mocks(
+            'full', 'THE image', '123', floated=False)
+
+        self.expect(paragraph.getShowTitle()).result(False)
+        self.expect(paragraph.getText()).result(
+            'Text')
+
+        book = self.providing_stub([IBook])
+        self.set_parent(paragraph, book)
+
+        self.replay()
+
+        view = getMultiAdapter((paragraph, request, layout), ILaTeXView)
+        latex = view.render()
+
+        self.assertNotIn(r'\end{wrapfigure}', latex)
+        self.assertIn(r'Text', latex)
+        self.assertIn(r'\includegraphics', latex)
